# -*- coding: utf-8 -*-

# FOGLAMP_BEGIN
# See: https://foglamp-foglamp-documentation.readthedocs-hosted.com
# FOGLAMP_END

""" Splunk North plugin"""

import aiohttp
import asyncio
import json
import uuid

from foglamp.common import logger
from foglamp.plugins.north.common.common import *

__author__ = "Mark Riddoch"
__copyright__ = "Copyright (c) 2019 Dianomic Systems"
__license__ = "Apache 2.0"
__version__ = "${VERSION}"

_LOGGER = logger.setup(__name__)


splunk_north = None
config = ""

_CONFIG_CATEGORY_NAME = "Splunk"
_CONFIG_CATEGORY_DESCRIPTION = "Splunk North Plugin"

_DEFAULT_CONFIG = {
    'plugin': {
         'description': 'Splunk North Plugin',
         'type': 'string',
         'default': 'splunk',
         'readonly': 'true'
    },
    'url': {
        'description': 'Destination URL',
        'type': 'string',
        'default': 'http://splunk:8088/services/collector/event',
        'order': '1',
        'displayName': 'URL'
    },
    "source": {
         "description": "Source of data to be sent on the stream. May be either readings or statistics.",
         "type": "enumeration",
         "default": "readings",
         "options": ["readings", "statistics"],
         'order': '2',
         'displayName': 'Source'
    },
    "token": {
        "description": "Splunk security token",
        "type": "string",
        "default": "42b66064-ee25-407c-897c-3ded78d21b38",
        'order': '3',
        'displayName': 'Splunk authorisation token'
    }
}


def plugin_info():
    return {
        'name': 'splunk',
        'version': '3.0.0-2-g5d0215e',
        'type': 'north',
        'interface': '1.0',
        'config': _DEFAULT_CONFIG
    }


def plugin_init(data):
    global splunk_north, config
    splunk_north = SplunkPlugin()
    config = data
    return config


async def plugin_send(data, payload, stream_id):
    try:
        is_data_sent, new_last_object_id, num_sent = await splunk_north.send_payloads(payload)
    except asyncio.CancelledError:
        pass
    else:
        return is_data_sent, new_last_object_id, num_sent


def plugin_shutdown(data):
    pass


class SplunkPlugin(object):
    """ North Splunk Plugin """

    def __init__(self):
        self.event_loop = asyncio.get_event_loop()

    async def send_payloads(self, payloads):
        is_data_sent = False
        last_object_id = 0
        num_sent = 0
        try:
            payload_block = list()

            for p in payloads:
                last_object_id = p["id"]
                read = dict()
                read["asset"] = p['asset_code']
                read["readings"] = p['reading']
                read["timestamp"] = p['user_ts']
                payload_block.append(read)

            num_sent = await self._send_payloads(payload_block)
            is_data_sent = True
        except Exception as ex:
            _LOGGER.exception("Data could not be sent, %s", str(ex))

        return is_data_sent, last_object_id, num_sent

    async def _send_payloads(self, payload_block):
        """ send a list of block payloads"""

        num_count = 0
        try:
            url = config['url']['value']
            connector = aiohttp.TCPConnector(verify_ssl=False)
            async with aiohttp.ClientSession(connector=connector) as session:
                result = await self._send(url, payload_block, session)
        except Exception as ex:
            _LOGGER.exception("Failed to send to Splunk, %s", str(ex))
            pass
        else: 
            num_count += len(payload_block)
        return num_count

    async def _send(self, url, payload, session):
        """ Send the payload, using provided socket session """
        splunk_token = config['token']['value']
        token = 'Splunk ' + splunk_token
        headers = {'Authorization': token}
        full_payload = {"event": payload}
        _LOGGER.info("Splunk payload: %s", json.dumps(full_payload))
        async with session.post(url, data=json.dumps(full_payload), headers=headers) as resp:
            result = await resp.text()
            status_code = resp.status
            if status_code in range(400, 500):
                _LOGGER.error("Bad request error code: %d, reason: %s", status_code, resp.reason)
                raise Exception
            if status_code in range(500, 600):
                _LOGGER.error("Server error code: %d, reason: %s", status_code, resp.reason)
                raise Exception
            return result
